@@ -93,6 +93,7 @@ class AWSpider:
     exposed_functions = []
     peers = {}
     peer_uuids = []
+    failed_peers = {}
     
     def __init__( self, 
                 aws_access_key_id, 
@@ -777,9 +778,10 @@ class AWSpider:
     def _checkPeersCallback( self, discovered ):
 
         existing_peers = set( self.peers.keys() )
+        failed_peers = set( self.failed_peers.keys() )
         discovered_peers = set( discovered.keys() )
-        new_peers = discovered_peers - existing_peers
-        old_peers = existing_peers - discovered_peers
+        new_peers = discovered_peers - ( existing_peers | failed_peers )
+        old_peers = ( existing_peers | failed_peers ) - discovered_peers
         
         deferreds = []
         
@@ -795,7 +797,12 @@ class AWSpider:
         
         for uuid in old_peers:
             logger.debug( "Removing peer %s" % uuid )
-            del self.peers[ uuid ]
+            
+            if uuid in self.peers:
+                del self.peers[ uuid ]
+                
+            if uuid in self.failed_peers:
+                del self.failed_peers[ uuid ]
         
         if len( new_peers ) > 0:
             if len(deferreds) > 0:
@@ -823,7 +830,7 @@ class AWSpider:
         
 #        print self.peers 
         logger.debug( "Peers updated to: %s" % self.peers )
-        
+        logger.debug( "Failed peers updated to: %s" % self.failed_peers )
         # Go through the peers and reorganize the group.
         pass
         
@@ -857,10 +864,18 @@ class AWSpider:
 
         if len(deferreds) > 0:
             d = DeferredList( deferreds, consumeErrors=True )
+            d.addCallback( self._verifyPeerCallback, uuid )
             return d
         else:
             return None
     
+    def _verifyPeerCallback( self, data, uuid ):
+
+        for row in data:
+            if row[0] == True:
+                return None
+        self.failed_peers[ uuid ] = True
+    
     def _verifyPeerLocalIPCallback( self, data, uuid, local_ip, port ):
         logger.debug( "Verified local IP for %s" % uuid )
         if uuid not in self.peers:
